﻿Type=StaticCode
Version=3.82
B4A=true
@EndOfDesignText@
'Code module
'Subs in this code module will be accessible from all modules.
Sub Process_Globals
	Type ActionItem (Text As String, Image As Bitmap, Value As Object)
	Type ActionList (Pnl As Panel, Canvas As Canvas _
		,Actions As List _
		,Visible As Boolean _
		,Title As String _
		,InAnimation As Animation _
		,OutAnimation As Animation _
		,Selected As Int _
		,ActivityName As String _
	)
	Dim ItemHeight As Int : ItemHeight = 45dip
	Dim TextSize As Float : TextSize = 16 'NOT dip.
	Dim TextLeftPos As Int : TextLeftPos = 40dip
	Dim TextColor As Int : TextColor = Colors.LightGray
	Dim TopPadding As Int : TopPadding = 5dip
	Dim ImageWidth As Int : ImageWidth = 16dip
	Dim ImageHeight As Int : ImageHeight = 16dip
	Dim ImageLeftPos As Int : ImageLeftPos = 10dip
	Dim ImageTopPos As Int : ImageTopPos = (ItemHeight - ImageHeight) / 2
	Dim SelectionColor As Int : SelectionColor = 0xFFFF8000
	Dim BackgroundImage As Object
End Sub

Sub InitializeList (Width As Int, Title As String, ActivityName As String) As ActionList
	BackgroundImage = CreateNinePatchDrawable("bg")
	Dim al As ActionList
	al.Initialize
	al.Actions.Initialize
	Dim bmp As Bitmap
	bmp.InitializeMutable(Width, 100%y)
	al.Canvas.Initialize2(bmp)
	al.Pnl.Initialize("ActionList")
	al.Pnl.Tag = al
	Dim bd As BitmapDrawable
	bd.Initialize(bmp)
	bd.Gravity = Bit.OR(Gravity.LEFT, Gravity.TOP)
	al.Pnl.Background = bd
	al.Title = Title
	al.InAnimation.InitializeAlpha("", 0, 1)
	al.InAnimation.Duration = 300
	al.OutAnimation.InitializeAlpha("", 1, 0)
	al.OutAnimation.Duration = 200
	al.Selected = -1
	al.ActivityName = ActivityName
	Return al
End Sub

Sub AddActionItem(AL As ActionList, Text As String, Image As Bitmap, ReturnValue As Object)
	Dim item As ActionItem
	item.Initialize
	item.Text = Text
	item.Image = Image
	item.Value = ReturnValue
	AL.Actions.Add(item)
End Sub

Sub Show (AL As ActionList, Activity As Activity, Left As Int, Top As Int)
	If AL.Visible Then 
		AL.InAnimation.Stop(AL.Pnl)
		AL.Pnl.RemoveView
	End If
	Dim h As Int
	h = (AL.Actions.Size + 1) * ItemHeight + TopPadding
	Activity.AddView(AL.Pnl, Left, Top, AL.Canvas.Bitmap.Width, h)
	If AL.Visible = False Then
		AL.InAnimation.Start(AL.Pnl)
	End If
	Dim c As Canvas
	c = AL.Canvas
	Dim r As Rect
	r.Initialize(0, 0, AL.Pnl.Width, AL.Pnl.Height)
	c.DrawRect(r, Colors.Transparent, True, 0)
	c.DrawDrawable(BackgroundImage, r)
	c.DrawText(AL.Title, AL.Pnl.Width / 2, TopPadding + ItemHeight / 2, Typeface.DEFAULT_BOLD, _
		 TextSize, TextColor, "CENTER")
	
	'Draw the items
	For i = 0 To AL.Actions.Size - 1
		drawItem(AL, i)
	Next
	AL.Pnl.Invalidate
	AL.Visible = True
End Sub

Sub Hide (AL As ActionList)
	If AL.Visible = False Then Return
	AL.Visible = False
	AL.OutAnimation.Start(AL.Pnl)
	AL.Pnl.RemoveView
End Sub
Sub Touch(Sdr As Object, Action As Int, X As Float, Y As Float)
	Dim p As Panel
	p = Sdr
	Dim al As ActionList
	al = p.Tag
	If Action <> 1 Then '1 is the value of Activity.ACTION_UP (not accessible from the code module)
		Dim currentItem As Int
		currentItem = Floor((Y - TopPadding - ItemHeight) / ItemHeight)
		If currentItem = al.Selected Then Return
		changeSelection(al, currentItem)
	Else If al.Selected >= 0 Then
		Hide(al)
		Dim item As ActionItem
		item = al.Actions.Get(al.Selected)
		CallSub2(al.ActivityName, "ActionList_Result", item.Value) 
		al.Selected = -1
	End If
End Sub
Sub drawItem(AL As ActionList, Index As Int)
	Dim item As ActionItem
	item = AL.Actions.Get(Index)
	If item.Image.IsInitialized Then
		Dim imageRect As Rect
		imageRect.Initialize(ImageLeftPos, 0, ImageLeftPos + ImageWidth, 0)
		imageRect.Top = (Index + 1) * ItemHeight + TopPadding + ImageTopPos
		imageRect.Bottom = imageRect.Top + ImageHeight
		AL.Canvas.DrawBitmap(item.Image, Null, imageRect)
	End If
	
	AL.Canvas.DrawText(item.Text, TextLeftPos, _
		(Index + 1 + 0.5) * ItemHeight + TopPadding + AL.Canvas.MeasureStringHeight(item.Text, Typeface.DEFAULT, TextSize) /2 _ 
			, Typeface.DEFAULT, TextSize, TextColor, "LEFT")
End Sub
Sub changeSelection(AL As ActionList, NewSelection As Int)
	Dim r As Rect
	Dim top As Int
	'delete the old selection
	If AL.Selected >= 0 Then
		Dim p As Path
		Dim y As Int
		y = (AL.Selected + 1) * ItemHeight + TopPadding
		p.Initialize(0,  y)
		p.LineTo(0, y + ItemHeight)
		p.LineTo(AL.Pnl.Width, y + ItemHeight)
		p.LineTo(AL.Pnl.Width, y)
		AL.Canvas.ClipPath(p)
		r.Initialize(0, 0, AL.Pnl.Width, AL.Pnl.Height)
		AL.Canvas.DrawRect(r, Colors.Transparent, True, 0)
		AL.Canvas.DrawDrawable(BackgroundImage, r) 
		drawItem(AL, AL.Selected)
		AL.Canvas.RemoveClip
	End If
	If NewSelection >= 0 AND NewSelection < AL.Actions.Size Then
		top = (NewSelection + 1) * ItemHeight + TopPadding + 3dip
		r.Initialize(3dip, top, AL.Pnl.Width - 3dip, top + ItemHeight - 6dip)
		AL.Canvas.DrawRect(r, SelectionColor, True, 0)
		drawItem(AL, NewSelection)
		AL.Selected = NewSelection
	Else
		AL.Selected = -1
	End If
	AL.Pnl.Invalidate
End Sub
Sub CreateNinePatchDrawable(ImageName As String) As Object
    Dim r As Reflector
    Dim package As String
    Dim id As Int
    package = r.GetStaticField("anywheresoftware.b4a.BA", "packageName")
    id = r.GetStaticField(package & ".R$drawable", ImageName)
    r.Target = r.GetContext
    r.Target = r.RunMethod("getResources")
    Return r.RunMethod2("getDrawable", id, "java.lang.int")
End Sub